/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"
#include <string>

static gboolean on_delete_event (GtkWidget*, GdkEvent*, gpointer);

// The magic all begins here...
int main (int argc, 
          char *argv[])
{
  if (argc > 1)
  {
    if (string (argv[1]) == "--help" || string (argv[1]) == "-?")
    {
      cout << "For help information, run 'man openldev'.\n";
      return EXIT_SUCCESS;
    }
    else if (string (argv[1]) == "--version")
    {
      string version = "OpenLDev " + string (VERSION)
                     + " (Chinstrap)\nCopyright (C) 2004-2006 Andrew Krause\n";
      cout << version;
      return EXIT_SUCCESS;
    }
  }
  
  gtk_init (&argc, &argv);
  gnome_program_init ("openldev", VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_NONE);
  
  string image_file = openldev_add_prefix ("/share/pixmaps/openldev/splash.png");
  GtkWidget *splash = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  GtkWidget *image = gtk_image_new_from_file (image_file.c_str());
  
  gtk_container_add (GTK_CONTAINER (splash), image);
  gtk_window_set_type_hint (GTK_WINDOW (splash), GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
  gtk_window_set_position (GTK_WINDOW (splash), GTK_WIN_POS_CENTER_ALWAYS);
  gtk_window_set_keep_above (GTK_WINDOW (splash), TRUE);
  gtk_window_set_decorated (GTK_WINDOW (splash), FALSE);
  gtk_window_set_has_frame (GTK_WINDOW (splash), FALSE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (splash), TRUE);
  gtk_widget_show_all (splash);
  
  while (gtk_events_pending())
    gtk_main_iteration ();
  
  GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  OpenLDev *openldev = openldev_new_with_window (window);
  string icon = openldev_add_prefix ("/share/pixmaps/openldev/openldev24.png");
  gtk_window_set_icon_from_file (GTK_WINDOW (window), icon.c_str(), NULL);

  g_signal_connect (G_OBJECT (window), "delete-event",
                    G_CALLBACK (on_delete_event), openldev);
  
  gtk_widget_destroy (splash);
  gtk_main();
	
  return EXIT_SUCCESS;
}

static gboolean on_delete_event (GtkWidget *widget, 
                                 GdkEvent *event, 
                                 gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  gboolean is_modified = FALSE;
  for (int i=0; i < openldev->notebook->tab_count; i++)
  {
    if (openldev->notebook->tabs[i]->modified)
      is_modified = TRUE;
  }
  
  if (!is_modified)
  {
    gtk_main_quit();
    return FALSE;
  }
  else if (openldev_question ("You have unsaved files! Are you.\nsure you want to close?"))
  {
    gtk_main_quit();
    return FALSE;
  }
  
  return TRUE;
}
