/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-environment.h"

void plugin_toggled (GtkCellRendererToggle*, gchar*, EnvironmentDialog*);
void plugin_chosen (GtkTreeSelection*, gpointer);
void openldev_environment_initialize_data (EnvironmentDialog*);
void openldev_environment_initialize_plugins (EnvironmentDialog*);
void openldev_environment_connect_signals (EnvironmentDialog*);

void openldev_environment_replace_tabs_changed (GtkToggleButton*, EnvironmentDialog*);
void openldev_environment_autoindent_changed (GtkToggleButton*, EnvironmentDialog*);
void openldev_environment_load_on_launch_changed (GtkToggleButton*, EnvironmentDialog*);
void openldev_environment_wrap_changed (GtkToggleButton*, EnvironmentDialog*);
void openldev_environment_split_words_changed (GtkToggleButton*, EnvironmentDialog*);
void openldev_environment_bracket_changed (GtkToggleButton*, EnvironmentDialog*);
void openldev_environment_right_margin_changed (GtkToggleButton*, EnvironmentDialog*);
void openldev_environment_tab_size_changed (GtkSpinButton*, EnvironmentDialog*);
void openldev_environment_autosave_changed (GtkSpinButton*, EnvironmentDialog*);
void openldev_environment_undo_actions_changed (GtkSpinButton*, EnvironmentDialog*);
void openldev_environment_error_changed (GtkColorButton*, EnvironmentDialog*);
void openldev_environment_warning_changed (GtkColorButton*, EnvironmentDialog*);
void openldev_environment_fontbutton_changed (GtkFontButton*, EnvironmentDialog*);

// Create the environment settings dialog and make sure all of fonts are correct.
EnvironmentDialog* 
openldev_env_dialog_new (OpenLDevPluginList *list,
                         EnvironmentSettings *settings)
{
  EnvironmentDialog *env = new EnvironmentDialog ();
  
  PangoFontDescription *fd = pango_font_description_from_string ("Sans Bold 10");
  string gladefile = openldev_add_prefix ("/share/openldev/glade/environment.glade");
  GladeXML *xml = glade_xml_new (gladefile.c_str(), NULL, NULL);

  env->list = list;
  env->settings = settings;
  env->autoindent = glade_xml_get_widget (xml, "auto_indent");
  env->autosave = glade_xml_get_widget (xml, "auto_save");
  env->bracket = glade_xml_get_widget (xml, "highlight_brackets");
  env->dialog = glade_xml_get_widget (xml, "dialog");
  env->error = glade_xml_get_widget (xml, "error_color");
  env->fontbutton = glade_xml_get_widget (xml, "fontbutton");
  env->load_on_launch = glade_xml_get_widget (xml, "load_on_launch");
  env->replace_tabs = glade_xml_get_widget (xml, "insert_spaces");
  env->right_margin = glade_xml_get_widget (xml, "display_margin");
  env->split_words = glade_xml_get_widget (xml, "split_words");
  env->tab_size = glade_xml_get_widget (xml, "tab_width");
  env->undo_actions = glade_xml_get_widget (xml, "undo_actions");
  env->warning = glade_xml_get_widget (xml, "warning_color");
  env->wrap = glade_xml_get_widget (xml, "text_wrapping");
  
  env->label_autoindent = glade_xml_get_widget (xml, "label_autoindent");
  env->label_bracket = glade_xml_get_widget (xml, "label_bracketmatch");
  env->label_compiler = glade_xml_get_widget (xml, "label_compiler");
  env->label_filemanagement = glade_xml_get_widget (xml, "label_filemanagement");
  env->label_font = glade_xml_get_widget (xml, "label_font");
  env->label_rightmargin = glade_xml_get_widget (xml, "label_rightmargin");
  env->label_tabstops = glade_xml_get_widget (xml, "label_tabstops");
  env->label_text_wrap = glade_xml_get_widget (xml, "label_textwrapping");
  env->label_undo = glade_xml_get_widget (xml, "label_undo");
  
  env->name = glade_xml_get_widget (xml, "name");
  env->author = glade_xml_get_widget (xml, "author");
  env->description = glade_xml_get_widget (xml, "description");
  env->plugins = glade_xml_get_widget (xml, "plugins");
  
  // Set the labels to have a bold font
  gtk_widget_modify_font (env->label_autoindent, fd);
  gtk_widget_modify_font (env->label_bracket, fd);
  gtk_widget_modify_font (env->label_compiler, fd);
  gtk_widget_modify_font (env->label_filemanagement, fd);
  gtk_widget_modify_font (env->label_font, fd);
  gtk_widget_modify_font (env->label_tabstops, fd);
  gtk_widget_modify_font (env->label_text_wrap, fd);
  gtk_widget_modify_font (env->label_undo, fd);
  gtk_widget_modify_font (env->label_rightmargin, fd);
  
  openldev_environment_initialize_data (env);
  openldev_environment_initialize_plugins (env);
  openldev_environment_connect_signals (env);
  
  return env;
}

void 
openldev_environment_initialize_data (EnvironmentDialog *env)
{
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (env->autosave), 
                             env->settings->auto_save);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (env->undo_actions), 
                             env->settings->undo);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (env->tab_size), 
                             env->settings->tab_size);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (env->autoindent), 
                                env->settings->auto_indent);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (env->bracket), 
                                env->settings->brackets);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (env->load_on_launch), 
                                env->settings->auto_load);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (env->replace_tabs), 
                                env->settings->replace_tabs);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (env->right_margin), 
                                env->settings->right_margin);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (env->split_words), 
                                env->settings->wrap_split);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (env->wrap), 
                                env->settings->wrap);

  GdkColor tmp = { 0, 0, 0, 0, },
           *color = gdk_color_copy (&tmp), 
           *color2 = gdk_color_copy (&tmp);
  
  gdk_color_parse (env->settings->error.c_str(), color);
  gdk_color_parse (env->settings->warning.c_str(), color2);
  gtk_color_button_set_color (GTK_COLOR_BUTTON (env->error), color);
  gtk_color_button_set_color (GTK_COLOR_BUTTON (env->warning), color2);

  env->fontname = env->settings->font_face + " " + env->settings->font_size;
  gtk_font_button_set_font_name (GTK_FONT_BUTTON (env->fontbutton), env->fontname.c_str());
}

void 
openldev_environment_initialize_plugins (EnvironmentDialog *env)
{
  GtkCellRenderer *renderer;
  GtkTreeIter row;
  GtkTreeModel *model = GTK_TREE_MODEL (gtk_tree_store_new (4, G_TYPE_BOOLEAN,
                                        G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_STRING));
                                                          
  gtk_tree_view_set_model (GTK_TREE_VIEW (env->plugins), model);

  // Create the columns
  renderer = gtk_cell_renderer_toggle_new ();
  g_object_set (renderer, "xalign", 0.5, NULL);
  g_signal_connect (renderer, "toggled", G_CALLBACK (plugin_toggled), env);
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (env->plugins), 0, "Enabled",
							      renderer, "active", 0, "visible", 1, "activatable", 2, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (env->plugins), 
                                   1, "Plugin Name", renderer, "text", 3, NULL);
  
  // Make sure to check whether or not to set the target active
  for (unsigned int i=0; i < env->list->plugins.size(); i++)
  {
    gtk_tree_store_append (GTK_TREE_STORE (model), &row, NULL);
    gtk_tree_store_set (GTK_TREE_STORE (model), &row, 
                        0, env->list->plugins[i]->enabled, 1, TRUE, 2, TRUE, 
                        3, env->list->plugins[i]->name.c_str(), -1);
  }
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (env->plugins));
  g_signal_connect (G_OBJECT (selection), "changed",
                    G_CALLBACK (plugin_chosen), (gpointer) env);
  gtk_tree_model_get_iter (model, &row, gtk_tree_path_new_from_string("0"));
  gtk_tree_selection_select_iter (selection, &row);
  g_object_unref (model);
}

// Connect the signals for all of the widgets
void 
openldev_environment_connect_signals (EnvironmentDialog *env)
{
  g_signal_connect (G_OBJECT (env->tab_size), "value-changed", G_CALLBACK 
                    (openldev_environment_tab_size_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->autosave), "value-changed", G_CALLBACK 
                    (openldev_environment_autosave_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->undo_actions), "value-changed", G_CALLBACK 
                    (openldev_environment_undo_actions_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->replace_tabs), "toggled", G_CALLBACK 
                    (openldev_environment_replace_tabs_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->autoindent), "toggled", G_CALLBACK 
                    (openldev_environment_autoindent_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->load_on_launch), "toggled", G_CALLBACK 
                    (openldev_environment_load_on_launch_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->wrap), "toggled", G_CALLBACK 
                    (openldev_environment_wrap_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->split_words), "toggled", G_CALLBACK 
                    (openldev_environment_split_words_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->bracket), "toggled", G_CALLBACK 
                    (openldev_environment_bracket_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->right_margin), "toggled", G_CALLBACK 
                    (openldev_environment_right_margin_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->fontbutton), "font-set", G_CALLBACK 
                    (openldev_environment_fontbutton_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->error), "color-set", G_CALLBACK 
                    (openldev_environment_error_changed), (gpointer) env);
  g_signal_connect (G_OBJECT (env->warning), "color-set", G_CALLBACK 
                    (openldev_environment_warning_changed), (gpointer) env);
}

string 
openldev_environment_dialog_get_color_string (GdkColor &color)
{
  char blue[3], green[3], red[3];
  snprintf (blue, sizeof(blue), "%X", color.blue);
  snprintf (red, sizeof(red), "%X", color.red);
  snprintf (green, sizeof(green), "%X", color.green);
  
  string clr = "#" + string(red);
  if (color.red == 0)
    clr += "0";
  clr += green;
  if (color.green == 0)
    clr += "0";
  clr += blue;
  if (color.blue == 0)
    clr += "0";
    
  return clr;
}

void 
openldev_environment_error_changed (GtkColorButton *widget,
                                    EnvironmentDialog *env)
{
  GdkColor color;
  gtk_color_button_get_color (GTK_COLOR_BUTTON (env->error), &color);
  string str = openldev_environment_dialog_get_color_string (color).c_str();
  openldev_environment_set_error (env->settings, str.c_str());
}

void 
openldev_environment_warning_changed (GtkColorButton *widget,
                                      EnvironmentDialog *env)
{
  GdkColor color;
  gtk_color_button_get_color (GTK_COLOR_BUTTON (env->warning), &color);
  string str = openldev_environment_dialog_get_color_string (color).c_str();
  openldev_environment_set_warning (env->settings, str.c_str());
}

void 
openldev_environment_fontbutton_changed (GtkFontButton *widget,
                                         EnvironmentDialog *env)
{
  string size = gtk_font_button_get_font_name (GTK_FONT_BUTTON (env->fontbutton));
  string name;
  
  while (size.find (" ", 0) != string::npos)
  {
    name += size.substr (0, size.find (" ", 0) + 1);
    size.erase (0, size.find (" ", 0) + 1);
  }
  
  openldev_environment_set_font_face (env->settings, name.c_str());
  openldev_environment_set_font_size (env->settings, size.c_str());
}

void 
openldev_environment_replace_tabs_changed (GtkToggleButton *togglebutton,
                                           EnvironmentDialog *env)
{
  gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (env->replace_tabs));
  openldev_environment_set_replace_tabs (env->settings, value);
}

void 
openldev_environment_autoindent_changed (GtkToggleButton *togglebutton,
                                         EnvironmentDialog *env)
{
  gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (env->autoindent));
  openldev_environment_set_auto_indent (env->settings, value);
}

void 
openldev_environment_load_on_launch_changed (GtkToggleButton *togglebutton,
                                             EnvironmentDialog *env)
{
  gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (env->load_on_launch));
  openldev_environment_set_auto_load (env->settings, value);
}

void 
openldev_environment_wrap_changed (GtkToggleButton *togglebutton,
                                   EnvironmentDialog *env)
{
  gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (env->wrap));
  openldev_environment_set_wrap (env->settings, value);
}

void 
openldev_environment_split_words_changed (GtkToggleButton *togglebutton,
                                          EnvironmentDialog *env)
{
  gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (env->split_words));
  openldev_environment_set_wrap_split (env->settings, value);
}

void 
openldev_environment_bracket_changed (GtkToggleButton *togglebutton,
                                      EnvironmentDialog *env)
{
  gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (env->bracket));
  openldev_environment_set_brackets (env->settings, value);
}

void 
openldev_environment_right_margin_changed (GtkToggleButton *togglebutton,
                                           EnvironmentDialog *env)
{
  gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (env->right_margin));
  openldev_environment_set_right_margin (env->settings, value);
}

void 
openldev_environment_tab_size_changed (GtkSpinButton *spinbutton,
                                       EnvironmentDialog *env)
{
  gint value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (env->tab_size));
  openldev_environment_set_tab_size (env->settings, value);
}

void 
openldev_environment_autosave_changed (GtkSpinButton *spinbutton,
                                       EnvironmentDialog *env)
{
  gint value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (env->autosave));
  openldev_environment_set_auto_save (env->settings, value);
}

void 
openldev_environment_undo_actions_changed (GtkSpinButton *spinbutton,
                                           EnvironmentDialog *env)
{
  gint value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (env->undo_actions));
  openldev_environment_set_undo (env->settings, value);
}

// A plugin was chosen from the list, so update the information about it
void 
plugin_chosen (GtkTreeSelection *selection,
               gpointer data)
{
  EnvironmentDialog *env = (EnvironmentDialog*) data;
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    int i = atoi (gtk_tree_path_to_string (gtk_tree_model_get_path (model, &iter)));
    gtk_label_set_text (GTK_LABEL (env->name), 
                        env->list->plugins[i]->name.c_str());
    gtk_label_set_text (GTK_LABEL (env->author), 
                        env->list->plugins[i]->author.c_str());
    gtk_label_set_text (GTK_LABEL (env->description), 
                        env->list->plugins[i]->description.c_str());
  }
}

// Toggle the plugin check button because the user said to do it and perform
// the respective action
void
plugin_toggled (GtkCellRendererToggle *renderer, 
                gchar *path_string, 
                EnvironmentDialog *env)
{
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (env->plugins));
  int ipath = atoi (path_string);
  GtkTreeIter iter;
  gboolean toggled;
  gint *column = (gint*) g_object_get_data (G_OBJECT (renderer), "column");

  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter, column, &toggled, -1);
  gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column, !toggled, -1);
  gtk_tree_path_free (path);
  
  if (!toggled)
  {
    env->list->plugins[ipath]->reload (NULL);
    openldev_environment_add_plugin (env->settings, env->list->plugins[ipath]->file.c_str());
  }
  else
  {
    env->list->plugins[ipath]->destroy (NULL);
    openldev_environment_remove_plugin (env->settings, env->list->plugins[ipath]->file.c_str());
  }
  
  env->list->plugins[ipath]->enabled = !toggled;   
}

GtkWidget* openldev_env_dialog_get_dialog (EnvironmentDialog *env)
{ return env->dialog;
}

int openldev_env_dialog_run (EnvironmentDialog *env)
{
  return gtk_dialog_run (GTK_DIALOG (env->dialog));
}
