/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-new-file.h"

void openldev_new_file_dialog_on_apply   (GtkWidget*, gpointer);
void openldev_new_file_dialog_on_cancel  (GtkWidget*, gpointer);

// Load the new file dialog, set the current folder to the project directory
// and connect the button signals.
NewFileDialog* openldev_new_file_dialog_new (gchar* location)
{
  NewFileDialog *newfile = new NewFileDialog ();
  
  newfile->file_location = location;
  string gladefile = openldev_add_prefix ("/share/openldev/glade/newfile.glade");
  GladeXML *xml = glade_xml_new (gladefile.c_str(), NULL, NULL);
  
  newfile->dialog = glade_xml_get_widget (xml, "dialog");
  newfile->cancel = glade_xml_get_widget (xml, "button_cancel");
  newfile->apply =  glade_xml_get_widget (xml, "button_ok");
  newfile->filechooser = glade_xml_get_widget (xml, "filechooser");
  newfile->name =   glade_xml_get_widget (xml, "filename");
  
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (newfile->filechooser),
                                       newfile->file_location.c_str());
  
  g_signal_connect (G_OBJECT (newfile->apply), "clicked", 
                    G_CALLBACK (openldev_new_file_dialog_on_apply), (gpointer) newfile);
  g_signal_connect (G_OBJECT (newfile->cancel), "clicked", 
                    G_CALLBACK (openldev_new_file_dialog_on_cancel), (gpointer) newfile);
  
  return newfile;
}

// Run the dialog and set the filename if the user specified one and accept
// was pressed. Otherwise set it to the string NULL.
void openldev_new_file_dialog_run (NewFileDialog *newfile, 
                                   string &file)
{
  int result = 0;
  
  while (result == 0)
  { result = gtk_dialog_run (GTK_DIALOG (newfile->dialog));
  }
  
  switch (result)
  {
    case (GTK_RESPONSE_ACCEPT):
    {
      file = newfile->file_location.c_str();
      break;
    }
    default:
      file = "NULL";
  }
}

// Emit a cancel signal
void openldev_new_file_dialog_on_cancel (GtkWidget *button, 
                                         gpointer data)
{
  NewFileDialog *newfile = (NewFileDialog*) data;
  gtk_dialog_response (GTK_DIALOG (newfile->dialog), GTK_RESPONSE_CANCEL);
}

// Apply was pressed and now we must handle it
void openldev_new_file_dialog_on_apply (GtkWidget *button, 
                                        gpointer data)
{
  NewFileDialog *newfile = (NewFileDialog*) data;
  string location = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(newfile->filechooser));
  string projectname = gtk_entry_get_text (GTK_ENTRY(newfile->name));
  
  // Create new file if no errors are found
  if (!(projectname.find(" ",0) != string::npos || projectname.empty()))
  {
    newfile->file_location = location + "/" + projectname;

    // Write new file
    openldev_write_file (newfile->file_location, "");
    gtk_dialog_response (GTK_DIALOG (newfile->dialog), GTK_RESPONSE_ACCEPT);
  }
  // Throw an error
  else 
    openldev_error ("You must specify a\nfile name!");
}
