/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-new-project.h"

#include <gdk/gdkkeysyms.h>
#include <sys/wait.h>
#include <unistd.h>

void openldev_new_project_dialog_populate_combobox (NewProjectDialog *project);
void openldev_new_project_dialog_on_apply  (GtkWidget*, gpointer);
void openldev_new_project_dialog_on_cancel (GtkWidget*, gpointer);
string openldev_new_project_dialog_check_for_errors (NewProjectDialog*);

gboolean openldev_new_project_dialog_space (GtkWidget*, GdkEventKey*, gpointer);

// Load the new project dialog, connect the signals and set the default project
// location to the user's home directory + /openldev/.
NewProjectDialog* openldev_new_project_dialog_new ()
{
  NewProjectDialog *project = new NewProjectDialog ();
  
  string gladefile = openldev_add_prefix ("/share/openldev/glade/newproject.glade");
  GladeXML *xml = glade_xml_new (gladefile.c_str(), NULL, NULL);

  project->dialog = glade_xml_get_widget   (xml, "dialog");
  project->combobox = glade_xml_get_widget (xml, "combobox");
  project->name = glade_xml_get_widget     (xml, "name");
  project->version = glade_xml_get_widget  (xml, "version");
  project->homepage = glade_xml_get_widget (xml, "homepage");
  project->email = glade_xml_get_widget    (xml, "email");
  project->author = glade_xml_get_widget   (xml, "author");
  project->location = glade_xml_get_widget (xml, "location");
  project->cancel = glade_xml_get_widget   (xml, "button_cancel");
  project->apply = glade_xml_get_widget    (xml, "button_ok");
  
  openldev_new_project_dialog_populate_combobox (project);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (project->location),
                                       g_get_home_dir());
  
  // Prevent spaces from being entered into the entries
  g_signal_connect (G_OBJECT (project->name), "key-press-event",
                    G_CALLBACK (openldev_new_project_dialog_space),
                    (gpointer) project);
  g_signal_connect (G_OBJECT (project->version), "key-press-event",
                    G_CALLBACK (openldev_new_project_dialog_space),
                    (gpointer) project);
  g_signal_connect (G_OBJECT (project->homepage), "key-press-event",
                    G_CALLBACK (openldev_new_project_dialog_space),
                    (gpointer) project);
  g_signal_connect (G_OBJECT (project->email), "key-press-event",
                    G_CALLBACK (openldev_new_project_dialog_space),
                    (gpointer) project);

  g_signal_connect (G_OBJECT (project->apply), "clicked", 
                    G_CALLBACK (openldev_new_project_dialog_on_apply),
                    (gpointer) project);
  g_signal_connect (G_OBJECT (project->cancel), "clicked", 
                    G_CALLBACK (openldev_new_project_dialog_on_cancel), 
                    (gpointer) project);
  
  return project;
}

// Run the new project dialog and return the project file that was created
// if the project was created.
GList* openldev_new_project_dialog_run (NewProjectDialog* project)
{
  GList *list = NULL;
  int result = 0;
  string text, command;
  
  while (result == 0)
  { result = gtk_dialog_run (GTK_DIALOG (project->dialog));
  }
  
  if (result == GTK_RESPONSE_ACCEPT)
  {
    string type = gtk_combo_box_get_active_text (GTK_COMBO_BOX (project->combobox));
    string location = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (project->location));
    string name = gtk_entry_get_text (GTK_ENTRY (project->name));
    string author = gtk_entry_get_text (GTK_ENTRY (project->author));
    string homepage = gtk_entry_get_text (GTK_ENTRY (project->homepage));
    string version = gtk_entry_get_text (GTK_ENTRY (project->version));
    string email = gtk_entry_get_text (GTK_ENTRY (project->email)), script;
    
    if (author.empty())
      author = "AUTHOR";
    if (homepage.empty())
      homepage = "HOMEPAGE";
    if (email.empty())
      email = "EMAIL";
    
    string directory = openldev_add_prefix ("/share/openldev/templates/"), comm, txt;
    unsigned int l;
    vector<string> TRUE_entries;
    vector<string> entries;
    const char* fn;
 
    GDir *dir = g_dir_open (directory.c_str(), 0, NULL);
    while ((fn = g_dir_read_name(dir)) != NULL)
      entries.push_back (fn);

    // Make sure they are actually plugins
    for (l=0; l < entries.size(); l++)
      if (entries[l].find(".sh", 0) != string::npos)
        TRUE_entries.push_back (entries[l]);
    entries.clear ();

    for (unsigned int i=0; i < TRUE_entries.size(); i++) 
    {
      gchar buff [8];
      comm = "sh " + directory + TRUE_entries[i] + " -n";
      txt = "\0";
      FILE *in = popen (comm.c_str(), "r");
      while (fgets (buff, sizeof (buff), in))
        txt += buff;
      pclose (in);
      
      if (type == txt)
      {
        script = comm.substr (3, comm.length() - 6);
        break;
      }
    }
  
    text = location + "/" + name + "/" + name + ".openldev";
    command = "sh " + script + " '" + location + "/" 
            + name + "/' '" + name + "' '" + author + "' '" + homepage 
            + "' '" + version + "' '" + email + "'";
    
    list = g_list_append (list, (gpointer) text.c_str());
    list = g_list_append (list, (gpointer) command.c_str());
  }
  
  gtk_widget_destroy (project->dialog);
  return list;
}

// This adds the different types of projects to the combobox
void openldev_new_project_dialog_populate_combobox (NewProjectDialog *project)
{ 
  string directory = openldev_add_prefix ("/share/openldev/templates/"), text, command;
  unsigned int l;
  vector<string> TRUE_entries;
  vector<string> entries;
  const char* fn;
  
  GDir *dir = g_dir_open (directory.c_str(), 0, NULL);
  while ((fn = g_dir_read_name(dir)) != NULL)
    entries.push_back (fn);
  
  // Make sure they are
  for (l=0; l < entries.size(); l++)
    if (entries[l].find(".sh", 0) != string::npos)
      TRUE_entries.push_back (entries[l]);
  entries.clear ();

  // Do the actual population of the types
	gtk_combo_box_remove_text (GTK_COMBO_BOX (project->combobox), 0);
	for (unsigned int i=0; i < TRUE_entries.size(); i++) 
  {
    gchar buff [8];
    command = "sh " + directory + TRUE_entries[i] + " -n";
    text = "\0";
    FILE *in = popen (command.c_str(), "r");
    while (fgets (buff, sizeof(buff), in))
      text += buff;
    pclose (in);
    
    gtk_combo_box_append_text (GTK_COMBO_BOX (project->combobox), (gchar*) text.c_str());
	}

  gtk_combo_box_set_active (GTK_COMBO_BOX (project->combobox), 0);
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
void openldev_new_project_dialog_on_cancel (GtkWidget *widget, 
                                            gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;
  gtk_dialog_response (GTK_DIALOG (ptr->dialog), GTK_RESPONSE_CANCEL);
}

// When the apply button is pressed, it is checked that the project name only
// contains letters and numbers and that a version and location is specified.
// The project is then created from a template and the Makefiles and configure
// file are edited. The OpenLDev project file is also created.
void openldev_new_project_dialog_on_apply (GtkWidget *widget, 
                                           gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;  
  string errortext = openldev_new_project_dialog_check_for_errors (ptr);
  
  // Create new project
  if (errortext == "NONE")
    gtk_dialog_response (GTK_DIALOG (ptr->dialog), GTK_RESPONSE_ACCEPT);
  // Throw an error
  else
    openldev_error (errortext);
}

string openldev_new_project_dialog_check_for_errors (NewProjectDialog* project)
{
  string ntxt = gtk_entry_get_text (GTK_ENTRY (project->name)),
         vtxt = gtk_entry_get_text (GTK_ENTRY (project->version)),
         ltxt = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (project->location));
  
  // Check for error when creating a project or file
  if (ntxt.empty())
    return "You must specify a\nproject name!";
  else if (!isalpha (ntxt[0]))
    return "The first character of a project\nname must be a letter!";
  else if (ntxt.find (" ",0) != string::npos)
    return "The project name must\nbe alphanumeric!";

  for (unsigned int i=1; i < ntxt.length(); i++)
    if (!isalnum (ntxt[i]) && ntxt[i] != '_')
      return "The project name must\nbe alphanumeric!";
      
  if (vtxt.empty())
    return "You must specify a\nversion number!";
  else if (ltxt.empty())
    return "You must specify a\nproject location!";
  else
    return "NONE";
}

gboolean 
openldev_new_project_dialog_space (GtkWidget *widget, 
                                   GdkEventKey *event,
                                   gpointer data)
{
  if (event->keyval == GDK_space ||
      event->keyval == GDK_KP_Space)
    return TRUE;
  
  return FALSE;
}
