/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-project.h"

void openldev_close_project (OpenLDev*);

// If a project is open, load the project information dialog.
void openldev_project_info (GtkMenuItem *menuitem,
                            gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty()) 
    openldev_error ("Please open or create\na project first!");
  else 
  {
    ProjectInfoDialog *dialog = openldev_project_info_dialog_new (openldev->project);
    openldev_project_info_dialog_run (dialog);
  }
}

// If a project is open, load the project properties dialog and set the
// changes specified by the user. Also, rewrite the project compile flags to
// the src/Makefile.am
void openldev_project_preferences (GtkMenuItem *menuitem,
                                   gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  else 
  {
    ProjectDialog *dialog = openldev_project_dialog_new (openldev->project);
    int result = openldev_project_dialog_run (dialog);
    
    if (result == GTK_RESPONSE_CLOSE)
      openldev->project = dialog->settings;
    
    openldev_project_settings_write_project_file (openldev->project);
    gtk_widget_destroy (dialog->dialog);
    delete dialog;
  }
}

// Close the project, clear all of the files and then disable all of the project
// menu and toolbar items
void openldev_close_project (OpenLDev *openldev)
{
  openldev_project_settings_close_project (openldev->project);
  openldev_populate_treeview (openldev);
  openldev_menu_file_save_all (openldev);
  openldev_file_manager_erase_all_tabs (openldev->notebook);
  openldev_environment_set_last_project (openldev->env, "");
    
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/MenuBar/File/FileClose"), FALSE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/MenuBar/Project"), FALSE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/MenuBar/Build"), FALSE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/ToolBar/BuildCompileToolbar"), FALSE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/ToolBar/BuildBuildToolbar"), FALSE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/ToolBar/BuildExecute"), FALSE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/ToolBar/BuildClean"), FALSE);
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager,
                            "/ToolBar/BuildStop"), FALSE);
}

// Close project file. First call the close project funtion and then erase
// the project files treeview, save all of the files and erase all of the tabs.
void openldev_file_close_project (GtkMenuItem *menuitem, 
                                  gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  gboolean is_modified = FALSE;
  
  for (int i=0; i < openldev->notebook->tab_count; i++)
  {
    if (openldev->notebook->tabs[i]->modified)
      is_modified = TRUE;
  }
  
  if (!is_modified)
    openldev_close_project (openldev);
  else if (openldev_question ("You have unsaved files! Are you.\nsure you want to close?"))
    openldev_close_project (openldev);
}
