/***************************************************************************
 *   Copyright (C) 2006 by Andrew Krause                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef OPENLDEV_MAKEFILE_PARSER_H
#define OPENLDEV_MAKEFILE_PARSER_H

#include "../openldev-utils.h"

#include <glib.h>
#include <iostream>
#include <vector>
#include <string>
#include <fstream>
using namespace std;

enum { TARGET_PROGRAM, TARGET_LIBRARY, TARGET_LTLIBRARY };

typedef struct _Makefile  Makefile;
typedef struct _MakefileVariable  MakefileVariable;
typedef struct _MakefileScript  MakefileScript;
typedef struct _MakefileTarget  MakefileTarget;
typedef struct _MakefilePrefix  MakefilePrefix;
typedef struct _MakefileBuffer  MakefileBuffer;

struct _MakefileBuffer
{
  vector<string> name;
  vector<vector<string> > content;
};

struct _MakefileVariable
{
  string name;
  vector<string> content;
};

struct _MakefileScript
{
  string header;
  vector<string> content;
};

struct _MakefilePrefix
{
  string name;
  string location;  
  vector<string> data;
};

struct _MakefileTarget
{
  int type;
  bool user_prefix;
  
  string name;
  string prefix;
  
  vector<string> sources;
  vector<string> ldflags;
  vector<string> ldadd;
};

struct _Makefile
{
  string file;

  vector<string> subdirs;
  vector<string> includes;
  
  vector<MakefileVariable*> variables;
  vector<MakefileScript*> scripts;
  vector<MakefileTarget*> targets;
  vector<MakefilePrefix*> prefixes;
};

MakefileBuffer* openldev_makefile_buffer_read (string);
Makefile* openldev_makefile_new (string);
void openldev_makefile_write (Makefile*);

vector<string> openldev_makefile_get_subdirs (Makefile*);
vector<string> openldev_makefile_get_sources (Makefile*);

#endif
