/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-settings.h"

#define PATH "/apps/openldev"

// List of keys that should be loaded
string keys [] = { "/auto_save", "/browser_width", "/output_height", "/width", "/height",
                   "/undo_actions", "/tab_size", "/auto_indent", "/auto_load", "/brackets",
                   "/line_numbers", "/replace_tabs", "/right_margin", "/wrap_enable",
                   "/wrap_split", "/error_color", "/warning_color", "/font_face",
                   "/font_size", "/last_project", "/recent_files", "/plugins" };

// The corresponding numbers to the keys
enum { AUTO_SAVE=0, BROWSER_WIDTH, OUTPUT_HEIGHT, WINDOW_WIDTH, WINDOW_HEIGHT,
       UNDO_ACTIONS, TAB_SIZE, AUTO_INDENT, AUTO_LOAD, BRACKETS, LINE_NUMBERS,
       REPLACE_TABS, RIGHT_MARGIN, WRAP_ENABLE, WRAP_SPLIT, ERROR_COLOR,
       WARNING_COLOR, FONT_FACE, FONT_SIZE, LAST_PROJECT, RECENT, PLUGINS };

void openldev_environment_load_settings_file (EnvironmentSettings*);
void openldev_environment_create_all (EnvironmentSettings*);
void openldev_environment_set_gboolean (EnvironmentSettings*, int, gboolean);
void openldev_environment_set_string (EnvironmentSettings*, int, const char*);
void openldev_environment_set_int (EnvironmentSettings*, int, int);

gint openldev_environment_get_int (EnvironmentSettings*, int);
gboolean openldev_environment_get_bool (EnvironmentSettings*, int);
gchar* openldev_environment_get_string (EnvironmentSettings*, int);
GSList* openldev_environment_get_list (EnvironmentSettings*, int);

// Create a new environment settings structure
EnvironmentSettings* openldev_environment_settings_new ()
{
  EnvironmentSettings* settings = new EnvironmentSettings();
  
  settings->client = gconf_client_get_default ();
  openldev_environment_load_settings_file (settings);
  
  return settings;
}

// Load all of the environment settings from GConf
void openldev_environment_load_settings_file (EnvironmentSettings *settings)
{
  if (!gconf_client_dir_exists (settings->client, PATH, NULL))
    gconf_client_add_dir (settings->client, PATH, GCONF_CLIENT_PRELOAD_NONE, NULL);
  
  openldev_environment_create_all (settings);
  
  settings->auto_save = openldev_environment_get_int (settings, 0);
  settings->browser_width = openldev_environment_get_int (settings, 1);
  settings->outputmanager_height = openldev_environment_get_int (settings, 2);
  settings->width = openldev_environment_get_int (settings, 3);
  settings->height = openldev_environment_get_int (settings, 4);
  settings->undo = openldev_environment_get_int (settings, 5);
  settings->tab_size = openldev_environment_get_int (settings, 6);
   
  settings->auto_indent = openldev_environment_get_bool (settings, 7);
  settings->auto_load = openldev_environment_get_bool (settings, 8);
  settings->brackets = openldev_environment_get_bool (settings, 9);
  settings->line_numbers = openldev_environment_get_bool (settings, 10);
  settings->replace_tabs = openldev_environment_get_bool (settings, 11);
  settings->right_margin = openldev_environment_get_bool (settings, 12);
  settings->wrap = openldev_environment_get_bool (settings, 13);
  settings->wrap_split = openldev_environment_get_bool (settings, 14);
   
  settings->error = openldev_environment_get_string (settings, 15);
  settings->warning = openldev_environment_get_string (settings, 16);
  settings->font_face = openldev_environment_get_string (settings, 17);
  settings->font_size = openldev_environment_get_string (settings, 18);
  settings->last_project = openldev_environment_get_string (settings, 19);
  
  settings->recent_files = openldev_environment_get_list (settings, 20);
  settings->plugins = openldev_environment_get_list (settings, 21);
}

int openldev_environment_get_int (EnvironmentSettings *settings, int i)
{ return gconf_client_get_int (settings->client, (PATH + keys[i]).c_str(), NULL);
}

gboolean openldev_environment_get_bool (EnvironmentSettings *settings, int i)
{ return gconf_client_get_bool (settings->client, (PATH + keys[i]).c_str(), NULL);
}

gchar* openldev_environment_get_string (EnvironmentSettings *settings, int i)
{ return gconf_client_get_string (settings->client, (PATH + keys[i]).c_str(), NULL);
}

GSList* openldev_environment_get_list (EnvironmentSettings *settings, int i)
{ return gconf_client_get_list (settings->client, (PATH + keys[i]).c_str(), 
                                GCONF_VALUE_STRING, NULL);
}

// Make sure that all of the settings already exist in GConf. If they don't, the
// default value is used.
void openldev_environment_create_all (EnvironmentSettings *settings)
{
  string key;
  int i;
  GConfValue *value;
  
  GSList *list = NULL;
  int default_int[] = { 10, 175, 175, 800, 600, 25, 2 };
  gboolean default_gboolean[] = { TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE };
  string default_str[] = { "#FF0000", "#0000CC", "Monospace", "10", "EMPTY" };
  
  // Check all integer elements
  for (i=0; i < 7; i++)
  {
    key = PATH + keys[i];
    value = gconf_client_get (settings->client, key.c_str(), NULL);
    if (value == NULL)
    {
      gconf_client_add_dir (settings->client, key.c_str(), GCONF_CLIENT_PRELOAD_NONE, NULL);
      gconf_client_set_int (settings->client, key.c_str(), default_int[i], NULL);
    }
  }
  
  // Check all boolean elements
  for (i=7; i < 15; i++)
  {
    key = PATH + keys[i];
    value = gconf_client_get (settings->client, key.c_str(), NULL);
    if (value == NULL)
    {
      gconf_client_add_dir  (settings->client, key.c_str(), GCONF_CLIENT_PRELOAD_NONE, NULL);
      gconf_client_set_bool (settings->client, key.c_str(), default_gboolean[i-7], NULL);
    }
  }
  
  // Check all string elements
  for (i=15; i < 20; i++)
  {
    key = PATH + keys[i];
    value = gconf_client_get (settings->client, key.c_str(), NULL);
    if (value == NULL)
    {
      gconf_client_add_dir (settings->client, key.c_str(), GCONF_CLIENT_PRELOAD_NONE, NULL);
      gconf_client_set_string (settings->client, key.c_str(), default_str[i-15].c_str(), NULL);
    }
  }
  
  // Check for list elements
  for (i=20; i < 22; i++)
  {  
    key = PATH + keys[i];
    value = gconf_client_get (settings->client, key.c_str(), NULL);
    if (value == NULL)
    {
      if (i == 21)
      {
        list = g_slist_append (list, (gpointer) "makefileeditor.plugin");
        list = g_slist_append (list, (gpointer) "insertinfo.plugin");
      }
      
      gconf_client_add_dir (settings->client, key.c_str(), GCONF_CLIENT_PRELOAD_NONE, NULL);
      gconf_client_set_list (settings->client, key.c_str(), GCONF_VALUE_STRING, list, NULL);
    }
  }
}

// These three functions set values of their type and then suggest
// as sync to GConf to make sure the changes are written.
void openldev_environment_set_gboolean (EnvironmentSettings *settings,
                                    int id, 
                                    gboolean value)
{
  gconf_client_set_bool (settings->client, (PATH + keys[id]).c_str(), 
                         value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

void openldev_environment_set_string (EnvironmentSettings *settings, 
                                      int id, 
                                      const char *value)
{
  gconf_client_set_string (settings->client, (PATH + keys[id]).c_str(), 
                           value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

void openldev_environment_set_int (EnvironmentSettings *settings, 
                                   int id, 
                                   int value)
{
  gconf_client_set_int (settings->client, (PATH + keys[id]).c_str(), 
                        value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

void openldev_environment_set_list (EnvironmentSettings *settings, 
                                    int id, 
                                    GSList *list)
{
  gconf_client_set_list (settings->client, (PATH + keys[id]).c_str(), 
                         GCONF_VALUE_STRING, list,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

// Functions that allow the user to change environment settings
void openldev_environment_set_auto_indent (EnvironmentSettings* settings, 
                                           gboolean value)
{
  settings->auto_indent = value;
  openldev_environment_set_gboolean (settings, AUTO_INDENT, value);
}

void openldev_environment_set_auto_load (EnvironmentSettings* settings, 
                                         gboolean value)
{
  settings->auto_load = value;
  openldev_environment_set_gboolean (settings, AUTO_LOAD, value);
}

void openldev_environment_set_brackets (EnvironmentSettings* settings, 
                                        gboolean value)
{
  settings->brackets = value;
  openldev_environment_set_gboolean (settings, BRACKETS, value);
}

void openldev_environment_set_line_numbers (EnvironmentSettings* settings, 
                                            gboolean value)
{
  settings->line_numbers = value;
  openldev_environment_set_gboolean (settings, LINE_NUMBERS, value);
}

void openldev_environment_set_replace_tabs (EnvironmentSettings* settings, 
                                            gboolean value)
{
  settings->replace_tabs = value;
  openldev_environment_set_gboolean (settings, REPLACE_TABS, value);
}

void openldev_environment_set_right_margin (EnvironmentSettings* settings, 
                                            gboolean value)
{
  settings->right_margin = value;
  openldev_environment_set_gboolean (settings, RIGHT_MARGIN, value);
}

void openldev_environment_set_wrap (EnvironmentSettings* settings, 
                                    gboolean value)
{
  settings->wrap = value;
  openldev_environment_set_gboolean (settings, WRAP_ENABLE, value);
}

void openldev_environment_set_wrap_split (EnvironmentSettings* settings, 
                                          gboolean value)
{
  settings->wrap_split = value;
  openldev_environment_set_gboolean (settings, WRAP_SPLIT, value);
}

void openldev_environment_set_auto_save (EnvironmentSettings* settings, 
                                         int value)
{
  settings->auto_save = value;
  openldev_environment_set_int (settings, AUTO_SAVE, value);
}

void openldev_environment_set_browser_width (EnvironmentSettings* settings, 
                                             int value)
{
  settings->browser_width = value;
  openldev_environment_set_int (settings, BROWSER_WIDTH, value);
}

void openldev_environment_set_height (EnvironmentSettings* settings, 
                                      int value)
{
  settings->height = value;
  openldev_environment_set_int (settings, WINDOW_HEIGHT, value);
}

void openldev_environment_set_outputmanager_height (EnvironmentSettings* settings, 
                                                    int value)
{
  settings->outputmanager_height = value;
  openldev_environment_set_int (settings, OUTPUT_HEIGHT, value);
}

void openldev_environment_set_tab_size (EnvironmentSettings* settings, 
                                        int value)
{
  settings->tab_size = value;
  openldev_environment_set_int (settings, TAB_SIZE, value);
}

void openldev_environment_set_undo (EnvironmentSettings* settings, 
                                    int value)
{
  settings->undo = value;
  openldev_environment_set_int (settings, UNDO_ACTIONS, value);
}

void openldev_environment_set_width (EnvironmentSettings* settings, 
                                     int value)
{
  settings->width = value;
  openldev_environment_set_int (settings, WINDOW_WIDTH, value);
}

void openldev_environment_set_error (EnvironmentSettings* settings, 
                                     const char* value)
{
  settings->error = value;
  openldev_environment_set_string (settings, ERROR_COLOR, value);
}

void openldev_environment_set_font_face (EnvironmentSettings* settings, 
                                         const char* value)
{
  settings->font_face = value;
  openldev_environment_set_string (settings, FONT_FACE, value);
}

void openldev_environment_set_font_size (EnvironmentSettings* settings, 
                                         const char* value)
{
  settings->font_size = value;
  openldev_environment_set_string (settings, FONT_SIZE, value);
}

void openldev_environment_set_last_project (EnvironmentSettings* settings, 
                                            const char* value)
{
  settings->last_project = value;
  openldev_environment_set_string (settings, LAST_PROJECT, value);
}

void openldev_environment_set_warning (EnvironmentSettings* settings, 
                                       const char* value)
{
  settings->warning = value;
  openldev_environment_set_string (settings, WARNING_COLOR, value);
}

void openldev_environment_add_recent_file (EnvironmentSettings *settings, 
                                           const char *value)
{
  for (unsigned int i=0; i < g_slist_length (settings->recent_files); i++)
    if (g_ascii_strcasecmp (value, (gchar*) g_slist_nth_data 
        (settings->recent_files, i)) == 0)
      return;
  
  gchar *new_value = g_strdup (value);
  settings->recent_files = g_slist_prepend (settings->recent_files, (gpointer) new_value);
  
  if (g_slist_length (settings->recent_files) > 5)
  {
    gpointer data = g_slist_nth_data (settings->recent_files, 5);
    settings->recent_files = g_slist_remove (settings->recent_files, data);
  }
  
  openldev_environment_set_list (settings, RECENT, settings->recent_files);
}

const char* openldev_environment_get_recent_short (EnvironmentSettings *settings,
                                                   int item)
{
  string temp = (gchar*) g_slist_nth_data (settings->recent_files, item);
  while (temp.find ("/", 0) != string::npos)
    temp.erase (0, temp.find ("/", 0) + 1);
  return temp.c_str();
}

void 
openldev_environment_remove_plugin (EnvironmentSettings *settings, 
                                    const char *plugin)
{
  for (unsigned int i=0; i < g_slist_length (settings->plugins); i++)
    if (g_ascii_strcasecmp (plugin, (gchar*) g_slist_nth_data (settings->plugins, i)) == 0)
    {
      gpointer data = g_slist_nth_data (settings->plugins, i);
      settings->plugins = g_slist_remove (settings->plugins, data);
    }

  openldev_environment_set_list (settings, PLUGINS, settings->plugins);
}

void 
openldev_environment_add_plugin (EnvironmentSettings *settings, 
                                 const char *plugin)
{
  for (unsigned int i=0; i < g_slist_length (settings->plugins); i++)
    if (g_ascii_strcasecmp (plugin, (gchar*) g_slist_nth_data (settings->plugins, i)) == 0)
      return;
  
  gchar *new_value = g_strdup (plugin);
  settings->plugins = g_slist_prepend (settings->plugins, (gpointer) new_value);
  
  openldev_environment_set_list (settings, PLUGINS, settings->plugins);
}

gboolean 
openldev_environment_get_plugin (EnvironmentSettings *settings, 
                                 const char *name)
{
  for (unsigned int i=0; i < g_slist_length (settings->plugins); i++)
    if (g_ascii_strcasecmp (name, (gchar*) g_slist_nth_data (settings->plugins, i)) == 0)
      return true;
  
  return false;
}
