/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-file-manager.h"

void openldev_file_manager_set_properties_one_tab (FileManager*, int, EnvironmentSettings*);
void openldev_file_manager_on_notebook_event (GtkNotebook*, gint, gpointer);
gboolean openldev_file_manager_on_textview_keypress (GtkWidget*, GdkEvent*, gpointer);
gboolean openldev_file_manager_on_textview_focus_out (GtkWidget*, GdkEventFocus*, gpointer);

// Make the file manager
FileManager* openldev_file_manager_new ()
{
  FileManager *manager = new FileManager ();
 
  manager->autolist = openldev_auto_list_new ();
  manager->notebook = gtk_notebook_new ();
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK(manager->notebook), TRUE);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK(manager->notebook), GTK_POS_TOP);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK(manager->notebook), TRUE);
  gtk_widget_show (manager->notebook);
  
  g_signal_connect (G_OBJECT(manager->notebook), "change_current_page",
                    G_CALLBACK(openldev_file_manager_on_notebook_event), (gpointer)manager);
  manager->tab_count = 0;
  
  return manager;
}

// Get the current text view
GtkWidget* openldev_file_manager_get_textview (FileManager *manager)
{ 
  g_assert (manager != NULL);
  int current = gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook));

  // Return the current textview
  if (manager->tab_count != 0)
    return manager->tabs[current]->textview;
  else
    return NULL;
}

// Get the text buffer from the current text view
GtkSourceBuffer* openldev_file_manager_get_textbuffer (FileManager *manager)
{ 
  g_assert (manager != NULL);
  return GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW
                            (openldev_file_manager_get_textview (manager))));
}

// Get the full filename of the current file
string openldev_file_manager_get_current_filename (FileManager *manager)
{ 
  g_assert (manager != NULL);
  return manager->tabs[gtk_notebook_get_current_page (GTK_NOTEBOOK (manager->notebook))]->fn;
}

// get the shortened filename of the current file
string openldev_file_manager_get_current_short_filename (FileManager *manager)
{
  g_assert (manager != NULL);
  int current = gtk_notebook_get_current_page (GTK_NOTEBOOK (manager->notebook));
  
  return gtk_label_get_text (GTK_LABEL (manager->tabs[current]->label));
}

// Get the id of the current tab that the user selected
int openldev_file_manager_get_current_tab (FileManager *manager)
{ 
  g_assert (manager != NULL);
  return gtk_notebook_get_current_page (GTK_NOTEBOOK (manager->notebook));
}

// Get the offset of the cursor in the current file
int openldev_file_manager_get_insert_offset (FileManager *manager)
{
  g_assert (manager != NULL);
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER (openldev_file_manager_get_textbuffer (manager));
  GtkTextIter iter;
  
  gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
  return gtk_text_iter_get_offset (&iter);
}

// Get the mime type of the specified file
string get_mime_type (string fn)
{ return gnome_vfs_get_mime_type (fn.c_str());
}

// Get the offset of the selection iterator in the current file
int openldev_file_manager_get_selection_offset (FileManager *manager)
{ 
  g_assert (manager != NULL);
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER (openldev_file_manager_get_textbuffer(manager));
  GtkTextIter iter;
  
  gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_selection_bound (buffer));
  return gtk_text_iter_get_offset(&iter);
}

// Check whether or not there is a file open
gboolean openldev_file_manager_get_file_open (FileManager *manager)
{
  g_assert (manager != NULL);
  if (gtk_notebook_get_n_pages (GTK_NOTEBOOK(manager->notebook)) > 0)
    return TRUE;
  
  return FALSE;
}

// Set a new filename to the tab
void openldev_file_manager_set_filename (FileManager *manager,
                                         const char* fn)
{
  g_assert (manager != NULL && fn != NULL);
  string file = fn;
  int current = gtk_notebook_get_current_page (GTK_NOTEBOOK(manager->notebook));
  manager->tabs[current]->fn = fn;
  
  while (file.find('/',0) != string::npos)
    file.erase (0, file.find('/',0) + 1);
  
  gtk_label_set_text (GTK_LABEL(manager->tabs[current]->label), file.c_str());
}

// Make sure all of the properties of every tab are up to dat
void openldev_file_manager_set_properties (FileManager* manager, 
                                           gboolean all_files, 
                                           EnvironmentSettings *env)
{
  g_assert (manager != NULL && env != NULL);
  
  if (all_files)
    for (int i=0; i < manager->tab_count; i++)
      openldev_file_manager_set_properties_one_tab (manager, i, env);
  else
    openldev_file_manager_set_properties_one_tab (manager, 
      gtk_notebook_get_current_page (GTK_NOTEBOOK(manager->notebook)), env);
}

// Make sure all of the properties of the specific tab are correct
void openldev_file_manager_set_properties_one_tab (FileManager *manager,
                                                   int tab,
                                                   EnvironmentSettings *env)
{
  g_assert (manager != NULL && env != NULL);
  g_assert (tab >= 0);

  GtkSourceView *view = GTK_SOURCE_VIEW (manager->tabs[tab]->textview);
  GtkSourceBuffer *buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer(GTK_TEXT_VIEW(manager->tabs[tab]->textview)));
  
  gtk_source_view_set_show_line_numbers (view, env->line_numbers);
  gtk_source_view_set_tabs_width (view, env->tab_size);
  gtk_source_view_set_auto_indent (view, env->auto_indent);
  gtk_source_view_set_show_margin (view, env->right_margin);
  gtk_source_view_set_insert_spaces_instead_of_tabs (view, env->replace_tabs);
  gtk_source_buffer_set_max_undo_levels (buffer, env->undo);
  gtk_source_buffer_set_check_brackets (buffer, env->brackets);

  // Set language specific values
  string mime_type = get_mime_type (manager->tabs[tab]->fn); 
  if (mime_type == "NONE")
    gtk_source_buffer_set_highlight (buffer, FALSE);
  else
  {
    gtk_source_buffer_set_highlight (buffer, TRUE);
    GtkSourceLanguagesManager *manager = gtk_source_languages_manager_new ();
    GtkSourceLanguage* language = gtk_source_languages_manager_get_language_from_mime_type (manager, mime_type.c_str());

    if (language)
      gtk_source_buffer_set_language (buffer, language);
  }
  
  if (env->wrap)
  {
    if (env->wrap_split)
      gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(view), GTK_WRAP_CHAR);
    else
      gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
  }
  else
      gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(view), GTK_WRAP_NONE);
}

// A new file was opened so we must add the tab for it
gboolean openldev_file_manager_add_tab (FileManager *manager,
                                        const char* file,
                                        EnvironmentSettings *env)
{
  g_assert (manager != NULL && file != NULL && env != NULL);

  string filename = file;
  FileTab *tab = openldev_file_tab_new (filename.c_str());
  tab->fn = filename;
  
  // Parse the filename to remove directory structure
  while (filename.find('/',0) != string::npos)
    filename.erase (0,filename.find('/',0)+1);

  // Check to make sure file is not already open
  for (int i=0; i < manager->tab_count; i++)
    if (tab->fn == manager->tabs[i]->fn)
    {
      gtk_notebook_set_current_page (GTK_NOTEBOOK(manager->notebook), i);
      return FALSE;
    }

  // Create temporary tab & initialize the objects
	tab->tab_id = manager->tab_count;

  // Add the tab to the notebook and notebook_tab's deque
	manager->tabs.push_back (tab);
  gtk_notebook_append_page (GTK_NOTEBOOK(manager->notebook),
                            manager->tabs[manager->tab_count]->swin,
                            manager->tabs[manager->tab_count]->hbox);
  
  // Do final initialization
  gtk_notebook_set_current_page (GTK_NOTEBOOK(manager->notebook), manager->tab_count);
	manager->tab_count++;
  return TRUE;
}

// Remove all of the tabs and tab information
void openldev_file_manager_erase_all_tabs (FileManager *manager)
{
  g_assert (manager != NULL);
  manager->tabs.erase (manager->tabs.begin(), manager->tabs.end());
  while (gtk_notebook_get_n_pages(GTK_NOTEBOOK(manager->notebook)) != 0)
    gtk_notebook_remove_page (GTK_NOTEBOOK(manager->notebook), 0);
  
  manager->tab_count = 0;
}

// MArk the current tab as modified or unmodified
void openldev_file_manager_mark_current_tab_modified (FileManager *manager,
                                                      gboolean modified,
                                                      int tab)
{
  gchar *markup;
  
  // If the file was modified
  if (modified)
  {
    markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", 
                      gtk_label_get_text (GTK_LABEL (manager->tabs[tab]->label)));
    gtk_label_set_markup (GTK_LABEL (manager->tabs[tab]->label), markup);
    manager->tabs[tab]->modified = true;
  }
  // If it was not modified
  else 
  {
    markup = g_markup_printf_escaped ("<span style=\"normal\">%s</span>", 
                      gtk_label_get_text (GTK_LABEL (manager->tabs[tab]->label)));
    gtk_label_set_markup (GTK_LABEL (manager->tabs[tab]->label), markup);
    manager->tabs[tab]->modified = false;
  }
  
  g_free (markup);
}

// Show the cursor on-screen. This is used for things such as the goto line function.
void openldev_file_manager_scroll_to_mark (FileManager *manager)
{
  g_assert (manager != NULL);
  
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER (openldev_file_manager_get_textbuffer(manager));
  GtkTextIter iter;
  gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert(buffer));
  gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(openldev_file_manager_get_textview(manager)), 
                                &iter, 0.0, FALSE, 0.0, 0.0);
}

// Set a new font for each of the tabs
void openldev_file_manager_modify_fonts (FileManager *manager,
                                         PangoFontDescription *fd)
{
  g_assert (manager != NULL && fd != NULL);
  for (int i=0; i < manager->tab_count; i++)
    gtk_widget_modify_font (manager->tabs[i]->textview, fd);
}

// Revert to the saved version of a file
void openldev_file_manager_revert_to_saved (FileManager *manager)
{
  g_assert (manager != NULL);
  int current = gtk_notebook_get_current_page (GTK_NOTEBOOK(manager->notebook));
  openldev_file_manager_mark_current_tab_modified (manager, FALSE, current);
  string text = openldev_read_file (manager->tabs[current]->fn);
  
  GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(manager->tabs[current]->textview));
  gtk_text_buffer_set_text (buffer, text.c_str(), -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(manager->tabs[current]->textview), 
                            GTK_TEXT_BUFFER(buffer));
}

// When an event happens in the notebook, make sure to grab focus.
void openldev_file_manager_on_notebook_event (GtkNotebook *widget,
                                              gint page,
                                              gpointer data)
{
  FileManager *ptr = (FileManager*) data;
  gint current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(widget));
  gtk_widget_grab_focus(ptr->tabs[current_page]->textview);
}

// Delete the tab for which the specific close button corresponds
void openldev_file_manager_on_delete_tab (GtkWidget *widget,
                                          gpointer data)
{
  FileManager *ptr = (FileManager*) data;
  
  for (int i=0; i < ptr->tab_count; i++)
    if (ptr->tabs[i]->button == widget)
    {
      // Make sure the user wants to close an unsaved file
      if (ptr->tabs[i]->modified)
        if (!openldev_question ("The file you are closing is not saved.\nAre you sure you want to close?"))
          return;

      // Remove the file and set the current page to the previous page
      vector<FileTab*>::iterator itr = ptr->tabs.begin();
      gtk_notebook_remove_page (GTK_NOTEBOOK(ptr->notebook), i);

      for (int count=0; count < i; count++)
        itr++;

      ptr->tab_count--;
      ptr->tabs.erase (itr);

	    if (ptr->tab_count != 0)
        gtk_notebook_set_current_page (GTK_NOTEBOOK(ptr->notebook), i-1);
      break;
    }    
}

// Set the tab width for when the file contains tabs instead of spaces. This
// function receives the font description, environment settings object and the
// current textview. It returns the tabarray from the textview
PangoTabArray* openldev_file_manager_get_tab_array (PangoFontDescription *fd, 
                                                    EnvironmentSettings *env,
                                                    GtkWidget *text_view)
{
  g_assert (env != NULL && fd != NULL && text_view != NULL);
  
  PangoTabArray *tab_array;
  PangoLayout *layout;
	string tab;
	int i=0, width, height;

  // Add spaces for the number of spaces per tab
	while (i < env->tab_size) 
  {
		tab += " ";
		i++;
	}

  // Create the pango layout & get the pixel width
  layout = gtk_widget_create_pango_layout (text_view, tab.c_str());
  pango_layout_set_font_description (layout, fd);
  pango_layout_get_pixel_size (layout, &width, &height);
 
  // Create the tab array from the width
  tab_array = pango_tab_array_new (1, TRUE);
  pango_tab_array_set_tab (tab_array, 0, PANGO_TAB_LEFT, width);

  return tab_array;
}
