/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"

// Initialization Functions
void openldev_initialize_objects (OpenLDev*);
void openldev_populate_window (OpenLDev*);
void openldev_create_icon_factory ();
void openldev_create_menus_and_toolbar (OpenLDev*);
void openldev_connect_signals (OpenLDev*);

// Signal Handler declarations
gboolean openldev_on_auto_save (gpointer);
gboolean openldev_on_button_press_event_treeview (GtkWidget*, GdkEventButton*, gpointer);
gboolean openldev_on_build_box_selected (GtkWidget*, GdkEventButton*, gpointer);
gboolean openldev_on_search_box_selected (GtkWidget*, GdkEventButton*, gpointer);
void openldev_on_resize (GtkWidget*, GdkEvent*, gpointer);
void openldev_on_resize_pane (GtkWidget*, GdkEvent*, gpointer);
void openldev_on_close_toolbars (GtkWidget*, OpenLDev*);
void openldev_on_goto_line (GtkWidget*, gpointer);
void openldev_on_find (GtkWidget*, gpointer);
void openldev_on_find_and_replace (GtkWidget*, gpointer);
void openldev_on_recent_file_activated (GtkMenuItem*, gpointer);

// Popup menu action handlers
void openldev_refresh_file_browser (GtkMenuItem*, gpointer);
void openldev_open_file_browser (GtkMenuItem*, gpointer);
void openldev_clear_build_output (GtkMenuItem*, gpointer);
void openldev_clear_search_output (GtkMenuItem*, gpointer);
void openldev_copy_output (GtkMenuItem*, gpointer);
void openldev_jump_to_build_output (GtkMenuItem*, gpointer);
void openldev_jump_to_search_output (GtkMenuItem*, gpointer);

#define NUM_ENTRIES 61
static GtkActionEntry entries[] = {
  { "File", NULL, "_File" },
  { "Edit", NULL, "_Edit" },
  { "View", NULL, "_View" },
  { "Search", NULL, "_Search" },
  { "Project", NULL, "_Project" },
  { "Build", NULL, "_Build" },
  { "Help", NULL, "_Help" },
  
  { "FileNewFile", GTK_STOCK_NEW, "_New File", "<control>n", "Create a new file", G_CALLBACK(openldev_file_new_file) },
  { "FileNewFileToolbar", GTK_STOCK_NEW, " New File", NULL, "Create a new file", G_CALLBACK(openldev_file_new_file) },
  { "FileNewProject", "NEWPROJECT", "New Project", NULL, "Create a new project", G_CALLBACK(openldev_file_new_project) },
  { "FileOpen", GTK_STOCK_OPEN, "_Open", "<control>o", "Open an existing file", G_CALLBACK(openldev_file_open) },
  { "FileOpenToolbar", GTK_STOCK_OPEN, " Open", NULL, "Open an existing file", G_CALLBACK(openldev_file_open) },
  { "FileSave", GTK_STOCK_SAVE, "_Save", "<control>s", "Save current file", G_CALLBACK(openldev_file_save) },
  { "FileSaveAs", GTK_STOCK_SAVE_AS, "Save _As", "<control><shift>s", "Save file by new name", G_CALLBACK(openldev_file_save_as) },
  { "FileSaveAll", NULL, "Save All", "<control><shift>l", "Save all open files", G_CALLBACK(openldev_file_save_all) },
  { "FileRevert", GTK_STOCK_REVERT_TO_SAVED, "_Reload File", NULL, "Revert to saved file", G_CALLBACK(openldev_file_revert) },
  { "FilePrint", GTK_STOCK_PRINT, "_Print", "<control>p", "Print the current file", G_CALLBACK(openldev_file_print) },
  { "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, "Print Preview", "<control><shift>p", "Preview the current file", G_CALLBACK(openldev_file_print_preview) },
  { "FileClose", GTK_STOCK_CLOSE, "_Close Project", "<control><shift>w", "Close current project", G_CALLBACK(openldev_file_close_project) },
  { "FileQuit", GTK_STOCK_QUIT, "_Quit", "<control>q", "Quit the application", G_CALLBACK(openldev_file_quit) },
  { "FileRecent1", NULL, "", NULL, "", NULL },
  { "FileRecent2", NULL, "", NULL, "", NULL },
  { "FileRecent3", NULL, "", NULL, "", NULL },
  { "FileRecent4", NULL, "", NULL, "", NULL },
  { "FileRecent5", NULL, "", NULL, "", NULL },

  { "EditUndo", GTK_STOCK_UNDO, "_Undo", "<control>z", "Undo last action", G_CALLBACK(openldev_edit_undo) },
  { "EditRedo", GTK_STOCK_REDO, "_Redo", "<control><shift>z", "Redo last undone action", G_CALLBACK(openldev_edit_redo) },
  { "EditCut", GTK_STOCK_CUT, "C_ut", "<control>x", "Cut selected text", G_CALLBACK(openldev_edit_cut) },
  { "EditCopy", GTK_STOCK_COPY, "_Copy", "<control>c", "Copy selected text", G_CALLBACK(openldev_edit_copy) },
  { "EditPaste", GTK_STOCK_PASTE, "_Paste", "<control>v", "Paste text from clipboard", G_CALLBACK(openldev_edit_paste) },
  { "EditSelectAll", NULL, "Select _All", "<control>a", "Select all text", G_CALLBACK(openldev_edit_select_all) },
  { "EditDeselect", NULL, "_Deselect", NULL, "Deselect selected text", G_CALLBACK(openldev_edit_deselect) },
  { "EditFind", GTK_STOCK_FIND, "_Find", "<control>f", "Open the Find toolbar", G_CALLBACK(openldev_edit_find) },
  { "EditFindNext", GTK_STOCK_FIND, "Find _Next", "<control>g", "Find next", G_CALLBACK(openldev_edit_find_next) },
  { "EditReplace", GTK_STOCK_FIND_AND_REPLACE, "Replace", "<control>r", "Find and replace", G_CALLBACK(openldev_on_find_and_replace) },
  { "EditGotoLine", GTK_STOCK_JUMP_TO, "_Goto Line", "<control>i", "Open Goto Line toolbar", G_CALLBACK(openldev_edit_goto_line) },
  { "EditSettings", GTK_STOCK_PREFERENCES, "Preferences", NULL, "Edit Environment settings", G_CALLBACK(openldev_edit_preferences) },
  
  { "ViewFiles", GTK_STOCK_JUSTIFY_LEFT, "_Project Files", NULL, "Show or hide project files", G_CALLBACK(openldev_view_browser) },
  { "ViewOutput", GTK_STOCK_JUSTIFY_LEFT, "_Build Output", NULL, "Show or hide build output", G_CALLBACK(openldev_view_output) },
  
  { "ProjectConfigure", GTK_STOCK_PREFERENCES, "_Configure Project", NULL, "Edit project properties", G_CALLBACK(openldev_project_preferences) },
  { "ProjectInfo", "PROJECTINFO", "_Project Info", NULL, "View project info", G_CALLBACK(openldev_project_info) },
  
  { "BuildCompile", "COMPILE_SMALL", "_Compile", "F9", "Compile current file", G_CALLBACK(openldev_build_compile) },
  { "BuildCompileToolbar", "COMPILE", "_Compile", NULL, "Compile current file", G_CALLBACK(openldev_build_compile) },
  { "BuildBuild", "BUILD_SMALL", "_Build", "F11", "Build project tree", G_CALLBACK(openldev_build_project) },
  { "BuildBuildToolbar", "BUILD", "_Build", NULL, "Build project tree", G_CALLBACK(openldev_build_project) },
  { "BuildExecute", GTK_STOCK_EXECUTE, "_Execute", "F3", "Execute project", G_CALLBACK(openldev_build_execute) },
  { "BuildAutogen", GTK_STOCK_CONVERT, "_Auto Generate", NULL, "Auto generate and configure", G_CALLBACK(openldev_build_auto_generate) },
  { "BuildConfigure", NULL, "Con_figure", NULL, "Run configure script", G_CALLBACK(openldev_build_configure) },
  { "BuildClean", GTK_STOCK_CLEAR, "_Make Clean", NULL, "Clean distribution tree", G_CALLBACK(openldev_build_clean) },
  { "BuildStop", GTK_STOCK_STOP, "_Stop", "F12", "Stop current action", G_CALLBACK(openldev_build_stop) },
  
  { "HelpDocs", GTK_STOCK_HELP, "_Documentation", "F1", "View project documentation", G_CALLBACK(openldev_help_documentation) },
  { "HelpBugs", NULL, "_Report Bugs", NULL, "Report bugs", G_CALLBACK(openldev_help_report_bug) },
  { "HelpAbout", GTK_STOCK_ABOUT, "_About", "<alt>a", "About OpenLDev", G_CALLBACK(openldev_help_about) },
  
  { "SearchOutputGo", GTK_STOCK_JUMP_TO, "Go To Match", NULL, "Go to line of match", G_CALLBACK(openldev_jump_to_search_output) },
  { "SearchOutputCopy", GTK_STOCK_COPY, "Copy", NULL, "Copy search text", G_CALLBACK(openldev_copy_output) },
  { "SearchOutputClear", GTK_STOCK_CLEAR, "Clear", NULL, "Clear serach output", G_CALLBACK(openldev_clear_search_output) },
  
  { "BuildOutputGo", GTK_STOCK_JUMP_TO, "Go To Error", NULL, "Go to line of error", G_CALLBACK(openldev_jump_to_build_output) },
  { "BuildOutputCopy", GTK_STOCK_COPY, "Copy", NULL, "Copy error text", G_CALLBACK(openldev_copy_output) },
  { "BuildOutputClear", GTK_STOCK_CLEAR, "Clear", NULL, "Clear build output", G_CALLBACK(openldev_clear_build_output) },
  
  { "OpenFileBrowser", GTK_STOCK_OPEN, "_Open File", NULL, "Open an existing file", G_CALLBACK(openldev_open_file_browser) },
  { "RefreshFileBrowser", GTK_STOCK_REFRESH, "Refresh", NULL, "Refresh file browser", G_CALLBACK(openldev_refresh_file_browser) }
};

#define NUM_ENTRIES_TOGGLE 3
static GtkToggleActionEntry toggle_entries[] = {
  { "ViewToolbar", NULL, "_Toolbar", NULL, "Show or hide toolbar", G_CALLBACK(openldev_view_toolbar), TRUE },
  { "ViewStatusBar", NULL, "_Status Bar", NULL, "Show or hide status bar", G_CALLBACK(openldev_view_statusbar), TRUE },
  { "ViewLineNumbering", NULL, "_Line Numbering", NULL, "Show or hide line numbers", G_CALLBACK(openldev_view_line_numbering), TRUE }
};

OpenLDev* openldev_new_with_window (GtkWidget *win)
{
  OpenLDev *openldev = new OpenLDev;
  
  // Set properties of main window
  openldev->window = win;
  gtk_window_set_title (GTK_WINDOW (openldev->window), 
                        g_strconcat ("OpenLDev ", VERSION, NULL));
  gtk_container_set_border_width (GTK_CONTAINER (openldev->window), 0);
  
  // Create settings objects
  openldev->project = openldev_project_settings_new ("");
  openldev->env = openldev_environment_settings_new ();
  openldev->editor = openldev_editor_new ();
  
  // Call functions to add main window widgets & show them all
  gtk_window_set_default_size (GTK_WINDOW (openldev->window), 
                               openldev->env->width,
                               openldev->env->height);
  
  openldev_initialize_objects (openldev);
  openldev->pluginlist = openldev_load_plugins_all (openldev);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (openldev->output->notebook), 0);
  openldev_manage_menu_items (openldev);
  openldev_initialize_gconf (openldev);
  
  if (openldev->env->auto_save > 0)
  {
    gint time = openldev->env->auto_save * 1000 * 60;
    g_timeout_add (time, GSourceFunc(openldev_on_auto_save), (gpointer) openldev);
  }
  
  // Load the last project or just set the directory to the home directory
  if (openldev->env->last_project != "EMPTY" && 
      openldev->env->last_project != "" && 
      openldev->env->auto_load)
    openldev_open_project (openldev, (gchar*) openldev->env->last_project.c_str());
  
  return openldev;
}

void openldev_initialize_objects (OpenLDev *openldev)
{
  openldev->vpaned = gtk_vpaned_new ();
  openldev->hpaned = gtk_hpaned_new ();
  openldev->vbox = gtk_vbox_new (FALSE, 0);
  openldev->hbox = gtk_hbox_new (FALSE, 0);
  openldev->statusbar = openldev_status_bar_new ();
  openldev->output = openldev_output_manager_new ();
  openldev->notebook = openldev_file_manager_new ();
  openldev->browser = openldev_browser_new ();
  openldev->findtoolbar = openldev_find_toolbar_new ();
  openldev->gototoolbar = openldev_goto_toolbar_new ();
  openldev->menus = openldev_menu_handle_new ();
  
  gtk_widget_set_size_request (openldev->browser->files->scrolled_window, 
                               openldev->env->browser_width, 0);
  gtk_widget_set_size_request (openldev->output->terminal->terminal, 0, 
                               openldev->env->outputmanager_height);
  openldev_browser_project_populate (openldev->browser->files, "NULL");
  
  // Add all of the widgets & show them
  openldev_populate_window (openldev);
  openldev_connect_signals (openldev);
  gtk_widget_show_all (openldev->window);
  
  openldev_find_toolbar_show_error (openldev->findtoolbar, FALSE);
  openldev_goto_toolbar_show_error (openldev->gototoolbar, FALSE);
  gtk_widget_hide (openldev->findtoolbar->hbox);
  gtk_widget_hide (openldev->gototoolbar->hbox);
}

void openldev_populate_window (OpenLDev *openldev)
{
  openldev_create_menus_and_toolbar (openldev);
  
  // Add menu & toolbar to main vbox
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->menus->menubar, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->menus->toolbar, FALSE, FALSE, 0);
  gtk_paned_pack1 (GTK_PANED(openldev->hpaned), openldev->browser->notebook, FALSE, TRUE);
  gtk_paned_pack2 (GTK_PANED(openldev->hpaned), openldev->notebook->notebook, TRUE, TRUE);
  gtk_paned_pack1 (GTK_PANED(openldev->vpaned), openldev->hpaned, TRUE, TRUE);
  gtk_paned_pack2 (GTK_PANED(openldev->vpaned), openldev->output->notebook, FALSE, TRUE);

  // Add scrolled_win3 to the second vbox and finish off with the statusbar
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->vpaned, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->gototoolbar->hbox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->findtoolbar->hbox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->statusbar->appbar, FALSE, FALSE, 0);

  // Add v' & h' boxes to the main window
  gtk_container_add (GTK_CONTAINER(openldev->window), openldev->vbox);
}

void openldev_connect_signals (OpenLDev *openldev)
{
  g_signal_connect (G_OBJECT (openldev->window), "event-after",
                    G_CALLBACK (openldev_on_resize), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->hpaned), "event-after",
                    G_CALLBACK (openldev_on_resize_pane), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->vpaned), "event-after",
                    G_CALLBACK (openldev_on_resize_pane), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->browser->files->treeview), "button-press-event",
                    G_CALLBACK (openldev_on_button_press_event_treeview), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->output->search->search_box), "button-press-event", 
                    G_CALLBACK (openldev_on_search_box_selected), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->output->build->build_box), "button-press-event",
                    G_CALLBACK (openldev_on_build_box_selected), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->gototoolbar->button_close), "clicked",
                    G_CALLBACK (openldev_on_close_toolbars), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->gototoolbar->button_goto_line), "clicked",
                    G_CALLBACK (openldev_on_goto_line), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->findtoolbar->button_close), "clicked",
                    G_CALLBACK (openldev_on_close_toolbars), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->findtoolbar->button_find), "clicked",
                    G_CALLBACK (openldev_on_find), (gpointer) openldev);
}

void openldev_populate_treeview (OpenLDev *openldev)
{ 
  openldev_browser_project_populate (openldev->browser->files,
           openldev_project_settings_get_location (openldev->project).c_str());
}

// This function creates a new icon factory that adds the stock menu/toolbar
// items for the compile, build, project files and about commands
void openldev_create_icon_factory ()
{
  GtkIconFactory *icon_factory = gtk_icon_factory_new ();
  
  // Add new stock menu items
  openldev_add_new_stock_item (icon_factory, "Compile", "COMPILE",
           g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/compile.png", NULL));
  openldev_add_new_stock_item (icon_factory, "Compile", "COMPILE_SMALL",
           g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/compile_sm.png", NULL));
  openldev_add_new_stock_item (icon_factory, "Build", "BUILD", 
           g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/build.png", NULL));
  openldev_add_new_stock_item (icon_factory, "Build", "BUILD_SMALL", 
           g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/build_sm.png", NULL));
  openldev_add_new_stock_item (icon_factory, "New Project", "NEWPROJECT", 
           g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/newproject.png", NULL));
  openldev_add_new_stock_item (icon_factory, "Project Info", "PROJECTINFO",
           g_strconcat (INSTALL_PREFIX, "/share/pixmaps/openldev/projectinfo.png", NULL));
  
  // Add items to default
  gtk_icon_factory_add_default (icon_factory);
  icon_factory = NULL;
}

void openldev_create_menus_and_toolbar (OpenLDev *openldev)
{
  // Create menu actions and icons
  openldev_create_icon_factory ();

  gtk_action_group_add_actions (openldev->menus->group, entries, NUM_ENTRIES, 
                                (gpointer)openldev);
  gtk_action_group_add_toggle_actions (openldev->menus->group, toggle_entries, 
                                       NUM_ENTRIES_TOGGLE, (gpointer)openldev);
  
  string ui1 = openldev_add_prefix ("/share/openldev/menus/mainmenu.ui");
  string ui2 = openldev_add_prefix ("/share/openldev/menus/toolbar.ui");
  string ui3 = openldev_add_prefix ("/share/openldev/menus/popup_menus.ui");
  GError *uierror;
  
  if (!gtk_ui_manager_add_ui_from_file (openldev->menus->uimanager, ui1.c_str(), &uierror))
    openldev_error ("<span weight='bold'>OpenLDev Error:</span>\n\nCannot load file <span style='italic'>mainmenu.ui</span>!");
  if (!gtk_ui_manager_add_ui_from_file (openldev->menus->uimanager, ui2.c_str(), &uierror))
    openldev_error ("<span weight='bold'>OpenLDev Error:</span>\n\nCannot load file <span style='italic'>toolbar.ui</span>!");
  if (!gtk_ui_manager_add_ui_from_file (openldev->menus->uimanager, ui3.c_str(), &uierror))
    openldev_error ("<span weight='bold'>OpenLDev Error:</span>\n\nCannot load file <span style='italic'>popup_menus.ui</span>!");
  
  gtk_ui_manager_insert_action_group (openldev->menus->uimanager, 
                                      openldev->menus->group, 0);
  gtk_window_add_accel_group (GTK_WINDOW (openldev->window), 
                              gtk_ui_manager_get_accel_group (openldev->menus->uimanager));
  
  openldev->menus->menubar = gtk_ui_manager_get_widget 
                             (openldev->menus->uimanager, "/MenuBar");
  openldev->menus->toolbar = gtk_ui_manager_get_widget 
                             (openldev->menus->uimanager, "/ToolBar");
  openldev->menus->popup_filebrowser = gtk_ui_manager_get_widget 
                             (openldev->menus->uimanager, "/FileBrowserMenu");
  openldev->menus->popup_searchoutput = gtk_ui_manager_get_widget 
                             (openldev->menus->uimanager, "/SearchOutputMenu");
  openldev->menus->popup_buildoutput = gtk_ui_manager_get_widget 
                             (openldev->menus->uimanager, "/BuildOutputMenu");
  
  GtkWidget *linenums, *close, *project, *build, *newfile, *open;
  
  linenums = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/View/ViewLineNumbering");
  close = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/File/FileClose");
  project = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/Project");
  build = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/Build");
  newfile = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/ToolBar/FileNewFileToolbar");
  open = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/ToolBar/FileOpenToolbar");
  
  gtk_widget_set_sensitive (close, FALSE);
  gtk_widget_set_sensitive (project, FALSE);
  gtk_widget_set_sensitive (build, FALSE);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (linenums), openldev->env->line_numbers);
  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (newfile), TRUE);
  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (open), TRUE);
  
  // Hide the recent files until they are needed or populate them
  for (int i=0; i < 5; i++)
  {
    string recent = "/MenuBar/File/FileRecent";
    gchar buff[2];
    sprintf (buff, "%d", i+1);
    recent += buff;
    
    if (g_slist_length (openldev->env->recent_files) <= i)
      gtk_widget_hide (gtk_ui_manager_get_widget (openldev->menus->uimanager, recent.c_str()));
    else
    {
      string name = string (buff) + ". " + string (openldev_environment_get_recent_short (openldev->env, i));
      gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (gtk_ui_manager_get_widget 
                          (openldev->menus->uimanager, recent.c_str())))),
                          name.c_str());
    }
    
    g_signal_connect (G_OBJECT (gtk_ui_manager_get_widget 
                      (openldev->menus->uimanager, recent.c_str())), "activate",
                      G_CALLBACK (openldev_on_recent_file_activated), 
                      (gpointer) openldev);
  }

  gtk_toolbar_set_style (GTK_TOOLBAR (openldev->menus->toolbar), GTK_TOOLBAR_BOTH_HORIZ);
  gtk_toolbar_set_show_arrow (GTK_TOOLBAR (openldev->menus->toolbar), FALSE);
  
  int i;
  for (i=0; i < NUM_ENTRIES; i++)
  {
    entries[i].name = NULL;
    entries[i].stock_id = NULL;
    entries[i].label = NULL;
    entries[i].accelerator = NULL;
    entries[i].tooltip = NULL;
  }
  for (i=0; i < NUM_ENTRIES_TOGGLE; i++)
  {
    toggle_entries[i].name = NULL;
    toggle_entries[i].stock_id = NULL;
    toggle_entries[i].label = NULL;
    toggle_entries[i].accelerator = NULL;
    toggle_entries[i].tooltip = NULL;
  }
}

// Quit the program
void openldev_file_quit (GtkMenuItem* menuitem, 
                         gpointer data) 
{ 
  OpenLDev *openldev = (OpenLDev*) data;
  gboolean is_modified = FALSE;
  string label;
  
  for (int i=0; i < openldev->notebook->tab_count; i++)
  {
    if (openldev->notebook->tabs[i]->modified)
      is_modified = TRUE;
  }
  
  if (!is_modified)
    gtk_widget_destroy (openldev->window);
  else if (openldev_question ("You have unsaved files! Are you.\nsure you want to close?"))
    gtk_widget_destroy (openldev->window);

}

void openldev_on_recent_file_activated (GtkMenuItem *item,
                                        gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  GtkWidget *label = gtk_bin_get_child (GTK_BIN (item));
  string text = gtk_label_get_text (GTK_LABEL (label));
  int choice = text[0] - 49;
  
  if (choice >= 0 && choice < 5)
    openldev_open_file (openldev, (gchar*) g_slist_nth_data 
                        (openldev->env->recent_files, choice));
}

// Autosave all of the files when called
gboolean openldev_on_auto_save (gpointer data)
{
  openldev_menu_file_save_all ((OpenLDev*)data);
  return TRUE;
}

// When the window is resized, this checks the width and height and stores
// them so the same size can be restored on restart.
void openldev_on_resize (GtkWidget *widget,
                         GdkEvent *event,
                         gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (event->type == GDK_CONFIGURE)
  {
    // Get the current height & width and store them
    int width=0, height=0;
    width = openldev->window->allocation.width;
    height = openldev->window->allocation.height;
    openldev_environment_set_width (openldev->env, width);
    openldev_environment_set_height (openldev->env, height);
  }
}

// When one of the panes is resized, this checks the width and height of each
// pane so the same sizes can be restored on restart.
void openldev_on_resize_pane (GtkWidget *widget,
                              GdkEvent *event,
                              gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;

  if (event->type == GDK_MOTION_NOTIFY)
  {   
    // Get the Project File & Output panes' width/height & store them
    openldev_environment_set_browser_width (openldev->env, 
        openldev->browser->files->scrolled_window->allocation.width);
    openldev_environment_set_outputmanager_height (openldev->env, 
        openldev->output->build->scrolled_window->allocation.height);
  }
}

// Catch button press event & open popup menu or handle selection. This function
// receives the button event. If it is a right click, then the popup menu is
// shown. If it is a double click, the selected file is opened
gboolean openldev_on_button_press_event_treeview (GtkWidget *widget,
                                              GdkEventButton *event,
                                              gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU (openldev->menus->popup_filebrowser), 
                    NULL, NULL, NULL, data, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gchar *fn;
      gtk_tree_model_get (model, &iter, 1, &fn, -1);
   
      // If the text is not a group title, open it
      string file = openldev->browser->files->base
                  + openldev->browser->files->current + fn;
      
      if (string(fn) == "..")
        openldev_browser_project_move_up (openldev->browser->files);
      else if (g_file_test (file.c_str(), G_FILE_TEST_IS_DIR))
        openldev_browser_project_move (openldev->browser->files, fn);
      else
        openldev_open_file (openldev, (gchar*) file.c_str());
      
      g_free (fn);
    }
  }
  else
    return FALSE;
  
  return TRUE;
}

// Catch the build box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and check if
// it is an error or warning. If it is, then the file containing the error or
// warning is opened. The cursor is then moved line of the error or warning
gboolean openldev_on_build_box_selected (GtkWidget *widget, 
                                     GdkEventButton *event, 
                                     gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU (openldev->menus->popup_buildoutput), 
                    NULL, NULL, NULL, data, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
  {
    openldev_jump_to_build_output (GTK_MENU_ITEM (gtk_menu_item_new()), data);    
    return TRUE;
  }
  
  return FALSE;
}

// Catch the search box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and then the 
// file containing the search text is opened. The cursor is then moved line of 
// the search result
gboolean openldev_on_search_box_selected (GtkWidget *widget,
                                          GdkEventButton *event,
                                          gpointer data)
{
  OpenLDev *openldev = (OpenLDev*)data;
  
  if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU (openldev->menus->popup_searchoutput), 
                    NULL, NULL, NULL, data, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
  {
    openldev_jump_to_search_output (GTK_MENU_ITEM (gtk_menu_item_new()), data);    
    return TRUE;
  }
  
  return FALSE;
}

// Close both toolbars. The find toolbar and the goto line toolbar.
void openldev_on_close_toolbars (GtkWidget *widget, 
                                 OpenLDev *openldev)
{
  gtk_widget_hide (openldev->gototoolbar->hbox);
  gtk_widget_hide (openldev->findtoolbar->hbox);
}

void openldev_on_goto_line (GtkWidget *widget,
                            gpointer data)
{ openldev_goto_to_line_number ((OpenLDev*)data, -1);
}

void openldev_on_find (GtkWidget *widget,
                       gpointer data)
{ openldev_find_next ((OpenLDev*)data);
}

void openldev_on_find_and_replace (GtkWidget *widget,
                                   gpointer data)
{ openldev_find_and_replace ((OpenLDev*)data);
}

void openldev_refresh_file_browser (GtkMenuItem *item,
                                    gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  openldev_browser_project_refresh (openldev->browser->files);
}

void openldev_open_file_browser (GtkMenuItem *item,
                                 gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                (openldev->browser->files->treeview));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gchar *fn;
    gtk_tree_model_get (model, &iter, 1, &fn, -1);
   
    // If the text is not a group title, open it
    string file = openldev->browser->files->base
                + openldev->browser->files->current + fn;
      
    if (string(fn) == "..")
      openldev_browser_project_move_up (openldev->browser->files);
    else if (g_file_test (file.c_str(), G_FILE_TEST_IS_DIR))
      openldev_browser_project_move (openldev->browser->files, fn);
    else
      openldev_open_file (openldev, (gchar*) file.c_str());
      
    g_free (fn);
  }
}

void openldev_jump_to_build_output (GtkMenuItem *item, 
                                    gpointer data)
{ 
  OpenLDev *openldev = (OpenLDev*) data;
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                (openldev->output->build->build_box));
  GtkTreeModel *model;
  GtkTreePath *path = gtk_tree_path_new ();
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    string dstr;
    gchar *file;
    int linenumber;
    gtk_tree_model_get (model, &iter, 0, &file, -1);
    
    // It the output is an error or a warning..
    if (openldev_is_error_or_warning (file))
    {
      gchar *dir;
      string temp = file;
      linenumber = openldev_get_file_name_and_line (temp) - 1;
      
      do
      {
        path = gtk_tree_model_get_path (model, &iter);
        gtk_tree_path_prev (path);
        gtk_tree_model_get_iter (model, &iter, path);
        gtk_tree_model_get (model, &iter, 0, &dir, -1);
        dstr = string(dir);
      } while (dstr.find ("make[", 0) == string::npos);
        
      dstr = dir;
      dstr.erase (0, dstr.find ("Entering", 0) + 20);
      dstr = dstr.substr (0, dstr.find ("'", 0));

      openldev_open_file (openldev, g_strconcat (dstr.c_str(), "/", temp.c_str(), NULL));
        
      // Move the cursor to the line of the error & scroll to the cursor
      GtkTextBuffer *buffer = GTK_TEXT_BUFFER (openldev_file_manager_get_textbuffer
                                              (openldev->notebook));
      GtkTextIter iter;

      gtk_text_buffer_get_iter_at_line (buffer, &iter, linenumber);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_insert(buffer), &iter);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
      openldev_file_manager_scroll_to_mark (openldev->notebook);
      gtk_widget_grab_focus (openldev_file_manager_get_textview(openldev->notebook));
      g_free (dir);
    }
      
    g_free (file);
  }
}

void openldev_jump_to_search_output (GtkMenuItem *item, 
                                     gpointer data)
{ 
  OpenLDev *openldev = (OpenLDev*) data;
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                (openldev->output->search->search_box));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gchar *file;
    int linenumber;
    gtk_tree_model_get (model, &iter, 0, &file, -1);
      
    // Open the file the error was found in
    string temp = file;
    linenumber = openldev_get_file_name_and_line (temp) - 1;
    file = g_strconcat (openldev_project_settings_get_location (openldev->project).c_str(), 
                        temp.c_str(), NULL);
    openldev_open_file (openldev, file);
    
    // Move the cursor to the line of the error & scroll to the cursor
    GtkTextIter iter;
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER (openldev_file_manager_get_textbuffer 
                                            (openldev->notebook));
      
    gtk_text_buffer_get_iter_at_line (buffer, &iter, linenumber);
    gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_insert(buffer), &iter);
    gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
    openldev_file_manager_scroll_to_mark (openldev->notebook);
    gtk_widget_grab_focus (openldev_file_manager_get_textview(openldev->notebook));
     
    g_free (file);
  }
}

void openldev_clear_build_output (GtkMenuItem *item, 
                                  gpointer data)
{ 
  OpenLDev *openldev = (OpenLDev*) data;
  openldev_output_build_clear (openldev->output->build);
}

void openldev_clear_search_output (GtkMenuItem *item, 
                                   gpointer data)
{ 
  OpenLDev *openldev = (OpenLDev*) data;
  openldev_output_search_clear (openldev->output->search);
}

void openldev_copy_output (GtkMenuItem *item, 
                           gpointer data)
{ openldev_menu_edit_copy ((OpenLDev*) data);
}
